@@ -5,7 +5,7 @@
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2005-08-04 (-openedDocument)
+ * modified: 2005-11-19
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -73,6 +73,7 @@
     [registrationDict setObject:@"NO"  forKey:@"expertMode"];
     [registrationDict setObject:@"2"   forKey:@"snap"];
     [registrationDict setObject:@"0"   forKey:@"lineWidth"];
+    [registrationDict setObject:@"YES"  forKey:@"selectByBorder"];
     [registrationDict setObject:@"20"  forKey:@"cacheLimit"];
 
     /* Import preferences defaults */
@@ -109,9 +110,11 @@
 - (NSString*)currentDirectory
 {   NSString	*cdir = [[self currentDocument] directory], *path;
 
-    path = (cdir && [cdir length]) ? cdir
-                                   : (haveOpenedDocument ? [((NSOpenPanel*)[NSOpenPanel openPanel]) directory]
-                                                         : vhfHomeDirectory());
+    if (cdir && [cdir length])
+        path = cdir;
+    else
+        path = (haveOpenedDocument ? [((NSOpenPanel*)[NSOpenPanel openPanel]) directory]
+                                   : vhfHomeDirectory());
     if (!path)
         return vhfHomeDirectory();
     return path;
@@ -236,7 +239,7 @@
 }
 
 /* created:  1993-01-??
- * modified: 2004-02-25
+ * modified: 2005-11-18
  */
 - (void)applicationDidFinishLaunching:(NSNotification *)notification
 {   NSApplication	*theApplication = [notification object];
@@ -259,15 +262,15 @@
             /* create HOME/Cenon */
             [fileManager createDirectoryAtPath:path attributes:nil];
             /* create HOME/Cenon/Projects */
-            [fileManager createDirectoryAtPath:[NSString stringWithFormat:@"%@/Projects", path]
+            [fileManager createDirectoryAtPath:vhfPathWithPathComponents(path, @"Projects", nil)
                                     attributes:nil];
             /* copy Cenon/Projects/.dir.tiff */
-            from = [NSString stringWithFormat:@"%@/Projects/.dir.tiff", localLibrary()];
-            to = [NSString stringWithFormat:@"%@/Projects/.dir.tiff", path];
+            from = vhfPathWithPathComponents(localLibrary(), @"Projects", @".dir.tiff", nil);
+            to = vhfPathWithPathComponents(path, @"Projects", @".dir.tiff", nil);
             [fileManager copyPath:from toPath:to handler:nil];
             /* copy Cenon/dir.tiff */
-            from = [NSString stringWithFormat:@"%@/.dir.tiff", localLibrary()];
-            to = [NSString stringWithFormat:@"%@/.dir.tiff", path];
+            from = vhfPathWithPathComponents(localLibrary(), @".dir.tiff", nil);
+            to = vhfPathWithPathComponents(path, @".dir.tiff", nil);
             [fileManager copyPath:from toPath:to handler:nil];
         }
     }
@@ -280,10 +283,6 @@
     /* load modules */
     [self loadModules];
 
-    /* AppCAM */
-    if ([self respondsToSelector:@selector(xyzInitParameter)])
-        [self performSelector:@selector(xyzInitParameter)];
-
     [self displayToolPanel:YES];
 
 #if !defined(GNUSTEP_BASE_VERSION) && !defined(__APPLE__)	// OpenStep 4.2
@@ -309,7 +308,7 @@
 
 - (id)listFromFile:(NSString*)fileName
 {   NSArray		*list = nil;
-    NSString		*path;
+    NSString		*path, *name;
     NSFileManager	*defaultManager = [NSFileManager defaultManager];
 
     //if ( [fileName rangeOfString:@".dxf" options:NSCaseInsensitiveSearch].length )
@@ -329,9 +328,10 @@
         /* load parameter file
          * 1st try it in the users home library then try it in /LocalLibrary
          */
-        path = [NSString stringWithFormat:@"%@/%@/%@.dev", userLibrary(), HPGLPATH, Prefs_HPGLParmsFileName];
+        name = [Prefs_HPGLParmsFileName stringByAppendingPathExtension:DEV_EXT];
+        path = vhfPathWithPathComponents(userLibrary(), HPGLPATH, name, nil);
         if ( ![defaultManager fileExistsAtPath:path] )
-        {   path = [NSString stringWithFormat:@"%@/%@/%@.dev", localLibrary(), HPGLPATH, Prefs_HPGLParmsFileName];
+        {   path = vhfPathWithPathComponents(localLibrary(), HPGLPATH, name, nil);
             if ( ![defaultManager fileExistsAtPath:path] )
             {	NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                 return nil;
@@ -354,9 +354,10 @@
         /* load parameter file
          * 1st try it in the users home library then try it in /LocalLibrary
          */
-        path = [NSString stringWithFormat:@"%@/%@/%@.dev", userLibrary(), GERBERPATH, Prefs_GerberParmsFileName];
+        name = [Prefs_GerberParmsFileName stringByAppendingPathExtension:DEV_EXT];
+        path = vhfPathWithPathComponents(userLibrary(), GERBERPATH, name, nil);
         if ( ![defaultManager fileExistsAtPath:path] )
-        {   path = [NSString stringWithFormat:@"%@/%@/%@.dev", localLibrary(), GERBERPATH, Prefs_GerberParmsFileName];
+        {   path = vhfPathWithPathComponents(localLibrary(), GERBERPATH, name, nil);
             if ( ![defaultManager fileExistsAtPath:path] )
             {	NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                 return nil;
@@ -386,11 +387,10 @@
                 {   [gerberImport release];
                     return nil;
                 }
-                path = [NSString stringWithFormat:@"%@/%@/%@.tab", userLibrary(), GERBERPATH, Prefs_GerberParmsFileName];
+                name = [Prefs_GerberParmsFileName stringByAppendingPathExtension:@"tab"];
+                path = vhfPathWithPathComponents(userLibrary(), GERBERPATH, name, nil);
                 if ( ![gerberImport loadApertures:path] )
-                {
-                    path = [NSString stringWithFormat:@"%@/%@/%@.tab",
-                           localLibrary(), GERBERPATH, Prefs_GerberParmsFileName];
+                {   path = vhfPathWithPathComponents(localLibrary(), GERBERPATH, name, nil);
                     if ( ![gerberImport loadApertures:path] )
                     {   NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                         [gerberImport release];
@@ -412,9 +412,10 @@
          */
         if ([(devFileName = Prefs_DINParmsFileName) length])
         {
-            path = [NSString stringWithFormat:@"%@/%@/%@.dev", userLibrary(), DINPATH, devFileName];
+            name = [devFileName stringByAppendingPathExtension:DEV_EXT];
+            path = vhfPathWithPathComponents(userLibrary(), DINPATH, name, nil);
             if ( ![defaultManager fileExistsAtPath:path] )
-            {   path = [NSString stringWithFormat:@"%@/%@/%@.dev", localLibrary(), DINPATH, devFileName];
+            {   path = vhfPathWithPathComponents(localLibrary(), DINPATH, name, nil);
                 if ( ![defaultManager fileExistsAtPath:path] )
                 {   NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                     return nil;
@@ -522,6 +523,12 @@
     NSString		*path;
     NSFileManager	*fileManager = [NSFileManager defaultManager];
 
+    if (!appIsRunning)	// give application a chance to load modules first
+    {
+        [self performSelector:@selector(openFile:) withObject:fileName afterDelay:0];
+        return YES;
+    }
+
     document = nil;
     if ( ![fileManager fileExistsAtPath:fileName] )
         return NO;
@@ -533,7 +540,7 @@
 
             if (!docu)
                 continue;
-            path = [NSString stringWithFormat:@"%@/%@", [docu directory], [docu name]];
+            path = vhfPathWithPathComponents([docu directory], [docu name], nil);
             if ( docu && [path isEqualToString:fileName] )
             {	[[[self windows] objectAtIndex:i] makeKeyAndOrderFront:self];
                 return YES;
@@ -565,8 +572,14 @@
     return NO;
 }
 
-/*
+/* Import files
+ * modified: 20005-09-25
  */
+typedef enum
+{   IMPORTTO_SELECTEDLAYER  = 0,
+    IMPORTTO_NEWLAYER       = 1,
+    IMPORTTO_EXISTINGLAYERS = 2
+} ImportToLayerSelection;
 - (void)import:sender
 {   NSArray		*fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
                                               @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
@@ -580,9 +593,9 @@
     [openPanel setAllowsMultipleSelection:NO];
 
     if ( [openPanel runModalForDirectory:openDir file:@"" types:fileTypes] )
-    {   int	createNewLayer = [iaPopup indexOfSelectedItem];
-        id	list;
-        DocView	*view = [[self currentDocument] documentView];
+    {   ImportToLayerSelection	importTo = [iaPopup indexOfSelectedItem];
+        id			list;
+        DocView			*view = [[self currentDocument] documentView];
 
         [openDir release];
         openDir = [[[openPanel filename] stringByDeletingLastPathComponent] retain];
@@ -590,29 +603,36 @@
 	fileName = [openPanel filename];
         list = [self listFromFile:fileName];
 
-        if (createNewLayer)
-            [view addList:list toLayerAtIndex:-1];
-        else
-        {   id	layerList = [view layerList];
-            int	i = [view indexOfSelectedLayer];
+        switch (importTo)
+        {
+            case IMPORTTO_NEWLAYER:
+                [view addList:list toLayerAtIndex:-1 /*replaceObjects:NO*/];
+                break;
+            case IMPORTTO_SELECTEDLAYER:
+            {   id	layerList = [view layerList];
+                int	i = [view indexOfSelectedLayer];
 
-            if ([[layerList objectAtIndex:i] editable])
-                [view addList:list toLayerAtIndex:i];
-            else
-                NSRunAlertPanel(@"", LAYERNOTEDITABLE_STRING, OK_STRING, nil, nil, NULL);
-            /*for (i=0; i<[layerList count]; i++)
                 if ([[layerList objectAtIndex:i] editable])
+                    [view addList:list toLayerAtIndex:i /*replaceObjects:NO*/];
+                else
+                    NSRunAlertPanel(@"", LAYERNOTEDITABLE_STRING, OK_STRING, nil, nil, NULL);
+                /*for (i=0; i<[layerList count]; i++)
+                    if ([[layerList objectAtIndex:i] editable])
                 {   [view addList:list toLayerAtIndex:i];
-                    break;
+                        break;
                 }
-            if (i>=[layerList count])
-                [view addList:list toLayerAtIndex:-1];*/
-        }
+                if (i>=[layerList count])
+                    [view addList:list toLayerAtIndex:-1];*/
+                break;
+            }
+            case IMPORTTO_EXISTINGLAYERS:
+                [view addList:list toLayerAtIndex:-2 /*replaceObjects:YES*/];
+         }
     }
     [openPanel setAccessoryView:nil];
 }
 
-/*
+/* modified: 2005-11-14
  */
 - (void)importASCII:sender
 {   NSArray		*fileTypes = [NSArray arrayWithObjects:@"txt", @"asc", @"tab", nil];
@@ -626,15 +646,19 @@
     [openPanel setAllowsMultipleSelection:NO];
     if ( [openPanel runModalForDirectory:openDir file:@"" types:fileTypes] )
     {   int		sort = [iaaRadio selectedColumn];
-        NSString	*tabName = ([iaaPopup indexOfSelectedItem]<1) ? nil : [NSString stringWithFormat:@"%@%@", [iaaPopup title], DICT_EXT], *string;
-        NSDictionary	*conversionDict = dictionaryFromFolder(CHARCONV_FOLDER, tabName);
+        NSString	*tabName, *string;
+        NSDictionary	*conversionDict = nil;
 
+        if ( [iaaPopup indexOfSelectedItem] >= 1 &&
+             (tabName = [NSString stringWithFormat:@"%@%@", [iaaPopup title], DICT_EXT]) )
+                conversionDict = dictionaryFromFolder(CHARCONV_FOLDER, tabName);
         [openDir release];
         openDir = [[[openPanel filename] stringByDeletingLastPathComponent] retain];
 
 	fileName = [openPanel filename];
         string = [NSString stringWithContentsOfFile:fileName];
-        [[[self currentDocument] documentView] importASCII:stringWithConvertedChars(string, conversionDict) sort:sort];
+        [[[self currentDocument] documentView] importASCII:stringWithConvertedChars(string, conversionDict)
+                                                      sort:sort];
     }
     [openPanel setAccessoryView:nil];
 }
@@ -741,7 +765,7 @@
 {   NSSavePanel	*savePanel = [NSSavePanel savePanel];
 
     /* we have to set the file type and accessory on every call,
-     * because it gets destroyed on a call of saveOutput !
+     * because it gets destroyed sometimes !
      */
     [savePanel setAccessoryView:[savePanelAccessory retain]];
     //[self changeSaveType:spaFormatPopUp];
@@ -783,8 +807,6 @@
         [infoPanel orderFront:self];
     }
 #endif
-    //if ( !dataPanel )
-    //    [self showDataPanel:self];
 
     if (![self openFile:path])
         NSRunAlertPanel(@"", CANTOPENFILE_STRING, OK_STRING, nil, nil);
@@ -1167,8 +1189,7 @@
           action == @selector(importASCII:) ||
           action == @selector(revertToSaved:) ||
           action == @selector(save:) ||
-          action == @selector(saveAs:) ||
-          action == @selector(saveOutput:)) &&
+          action == @selector(saveAs:)) &&
          ![self currentDocument] )
         return NO;
 
